package Logrep::Common;

#############################################################################
#
# LOGREP - Logfile Extraction And Reporting System                          
# Copyright (C) 2002-2003 Tevfik Karaglle                                   
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
#
#############################################################################

#########
# LOGREP - Log reporting system common library, jan 02, TK

=head1 NAME

Logrep::Common - Common library for logrep modules

=head1 SYNOPSIS

    use Logrep::Common;

    SetCommonParameters ();
    GenerateReport ();
    GenerateTables ();


    Exported variables :
        
        $config - pointer to logrep configuration object
        $dieopenmsg - common error message during file open
        $dieclosemsg - common error message during file close
        @dim - array containing attribute list. Must be set.
        $startdate - date from first log line (used in reports.)
	$enddate - date from last log line (used in reports.)
	%monthnr - lookup table (from MMM to month number)
	$n - number of lines in the processes log file. Must be set.

=head1 DESCRIPTION

This module contains common reporting, counting and analysis functions used by 
logrep parsing modules.

=cut

###########################################################

use strict;
use Exporter;
use FindBin qw($Bin);
use Cwd;

use lib "$Bin/..";
use Logrep::Config qw($config);
use GD::Graph::bars;
use Net::DNS;
use CGI qw/*table/;

# Module definition globals
#
our $VERSION = "1.0.0";
our @ISA = qw(Exporter);
our @EXPORT = qw (
	&SetCommonParameters
	&GenerateReport
	&GenerateTables
	&GenerateHtmlHeader
	&GenerateHtmlTrailer
	$config
	$dieopenmsg
	$dieclosemsg
	@dim
	$startdate
	$logfilename
	$enddate
	%monthnr
	$n
	%lookup
);

our $styleCode = "
A:hover {color : Green;}
H1 {font-family : Verdana, Geneva, Arial, Helvetica, sans-serif; font-size : x-large;}
H2 {font-family : Verdana, Geneva, Arial, Helvetica, sans-serif; font-size : large;}
H3 {font-family : Verdana, Geneva, Arial, Helvetica, sans-serif; font-size : medium;}
BODY {font-family : Verdana, Geneva, Arial, Helvetica, sans-serif;
font-size : 12; color : #000080; background-color : #E6E6FA;}
TABLE {font-family : Verdana, Geneva, Arial, Helvetica, sans-serif; font-size : 12; color : #000080;}
";

our @EXPORT_OK = ();
our %EXPORT_TAGS = ();

our ( 
	%monthnr, $dieopenmsg, $dieclosemsg, $q, $indexpage, $ROOT, %argument,
	$curroot, $parent, $webdir, $logfilename, $verbose, $ndimension, $subsetargs,
	$maxdim, @dim, @DimOrg, $startdate, $enddate, $starttime, $previnterval, $elapsed,
	$htmlheader, $n, $pagecount, $subsetvector, $lookupargs, $lupdns, %lookup, $lookupcount,
	%dnscache, $chits, $qok, $qerrs, $noip, $gcount, $gbytes, %gdim, $htmltrailer
);

%monthnr = (
	"Jan" => "01",
	"Feb" => "02",
	"Mar" => "03",
	"Apr" => "04",
	"May" => "05",
	"Jun" => "06",
	"Jul" => "07",
	"Aug" => "08",
	"Sep" => "09",
	"Oct" => "10",
	"Nov" => "11",
	"Dec" => "12"
);


$dieopenmsg = "Can't open file $!";
$dieclosemsg = "Can't close file $!";

@dim = ();
%lookup = ();
%dnscache = ();
$lookupcount = $pagecount = 0;
$gcount = 1;

$q = new CGI ('');

no strict 'refs';

###############################
# SetCommonParameters : Sets common parameters for reporting modules
#
sub SetCommonParameters {
	
	$curroot = cwd;
	$indexpage = "index.html";
	$ROOT = "ROOT";
	@DimOrg = @dim;	# keep original copy of attribute list
	# level 0 hash
	$parent = { };

	$webdir = $config->webdir;
	$logfilename = $config->logfile;
	$verbose = $config->verbose;
	$ndimension = $config->dimension;
	$subsetargs = $config->subset;
	$subsetvector = $config->singlevector;
	$lookupargs = $config->lookup;

	VerifyParameters ();
	CheckSubsetList ();

	CheckLookupList ();

	# make a hash list out of attribute array
	foreach my $t (@dim) { $gdim {$t} = 1 };
	
	$startdate = $enddate = "";
		
	ProcessVerboseInfo ('init') if $verbose;	
}

###############################
# ProcessVerboseInfo : Generates/initializes verbose info
# One parameter : vcmd
#	'init' - initilaize counters
# 	'count', 'lookup', 'page' - process counter
# one optional parameter:
#	vcnt - counter value
#
sub ProcessVerboseInfo  {

	my ($vcmd, $vcnt) = @_;

	# Initialize counters	
	if ($vcmd eq "init") {	
		$starttime = time;
		$previnterval = 0;
		return;
	}

	$elapsed = time - $starttime; # elapsed time in seconds
		
	# Print only once for each verbose interval
	if (($elapsed != $previnterval) && (($elapsed % $config->FEEDBACKSEC) == 0)) {
		my $ltxt = ($vcmd eq 'count') ? 'lines' : ($vcmd eq 'lookup') ? 'lookups' : 'pages';
		printf "%d (%d $ltxt per sec)\n", $vcnt, $vcnt / $elapsed;
		$previnterval = $elapsed;
	}	
}

###############################
# GenerateTables : Generates counter tables
#	two parameters - count and bytes
#
sub GenerateTables {

	($gcount, $gbytes) = @_;

	if ($subsetvector) {
		UpdateVectorCounters ();
	} else {
		UpdateCounters (0, $ROOT, $parent);
	}

	ProcessVerboseInfo ('count', $.) if $verbose;
}

###############################
# UpdateCounters: Updates counter tables recursively
# 	two parameters -
#		dim - current dimension
#		currattr - current attribute,
#		parent - parent hash pointer
#
sub UpdateCounters {

	my ($dim, $currattr, $parent) = @_;

	my $newparent = $parent->{$currattr}{$$currattr};
	
	if (!defined $newparent) {
		
		$newparent = $parent->{$currattr}{$$currattr} = 
			{
				"count" => $gcount,	# Number of occurences
				"size" => $gbytes,
				"directory" => sprintf ("%08d", int (rand($config->MAX_RANDIR))), # Random directory name
				"ref" => { }	# Reference to structures at lower dimensions
			};				
	} else {		
		$newparent->{"count"} += $gcount;
		$newparent->{"size"} += $gbytes;	
	}

	return if ($dim == $ndimension);
	
	$dim++;
		
	for my $attrval (keys %gdim) {

		next if $gdim {$attrval} == 0;
	
		$gdim {$attrval} = 0;
		UpdateCounters ($dim, $attrval, $newparent->{"ref"});		
		$gdim {$attrval} = 1;		
	}

}

###############################
# UpdateVectorCounters: Updates counter tables for a vector 
# 	Two parameters -
#
sub UpdateVectorCounters {

	my $currattr = $ROOT;	
	my $workparent = $parent;	
	my $i = 0;
	
	do {	
		my $newparent = $workparent->{$currattr}{$$currattr};
	
		if (!defined $newparent) {
		
			$newparent = $workparent->{$currattr}{$$currattr} = 
				{
					"count" => $gcount,	# Number of occurences
					"size" => $gbytes, 
					"directory" => sprintf ("%08d", int (rand($config->MAX_RANDIR))), # Random directory name
					"ref" => { }	# Reference to structures at lower dimensions
				};
				
		} else {		
			$newparent->{"count"} += $gcount;
			$newparent->{"size"} += $gbytes;		
		}
		
		$workparent = $newparent->{"ref"};
		$currattr = $dim [$i++];
				
	} until ($i > $ndimension);
		
}

###############################
# GenerateReport: Generates logrep report based on counting tables
#
sub GenerateReport {

	my ($sumtitle, $suminfo) = @_;

	ProcessVerboseInfo ('init') if $verbose;

	# Run a lookup update if some lookups are defined
	if (scalar %lookup > 0) {
		LookupUpdate (1, $parent->{$ROOT}{$$ROOT}{"ref"}); 
	}
	# Generate HTML header
	$htmlheader = GenerateHtmlHeader ($sumtitle,$suminfo);

	# Generate HTML trailer
	$htmltrailer = GenerateHtmlTrailer ();

	ProcessVerboseInfo ('init') if $verbose;
	
	# Write details pages
	WriteDetailsPages (1, $parent->{$ROOT}{$$ROOT}, $webdir, '');
	
}

###############################
# LookupUpdate : 
#	Updates  values in counting tables by values from lookup functions
#	two parameters -
#		dim - current dimension
#		parent - pointer to count tables for the current level
#
sub LookupUpdate {

	my ($dim, $parent) = @_;

	ProcessVerboseInfo ('lookup', $lookupcount) if $verbose;

	$dim++;

	# lookup for specified lookup parameters
	foreach my $currattr (keys %$parent) {

		my $ltype = $lookup {$currattr};
		my $currattrval = $parent->{$currattr};
		my $xcount = 0;	# number of lookup-ed values, used by algorithm for throttling

		# Perform lookup update for values of the lookup-attribute
		foreach my $currattr2 (sort { $currattrval->{$b}{"count"} <=> $currattrval->{$a}{"count"} } keys %$currattrval) {
			
			last if ($config->MAX_X_COUNT > 0 && $xcount == $config->MAX_X_COUNT); # Throttling lookup			
			$xcount++;
				
			my $newparent = $currattrval->{$currattr2};

			# lookup value, can be undefined or null string (if lookup is defined)
			if (defined $ltype) {
				$newparent->{"lookup"} = GetLookupValue ($currattr2, $ltype);
			}

			next if ($dim > $ndimension);
	
			LookupUpdate ($dim, $newparent->{"ref"}); 	# Traverse recursively		
		}

	}

}

###############################
# GetLookupValue : 
#	Returns a lookup value from a lookup method. Undef if something is wrong or incomplete
#	two parameters -
#		lattr - attribute value to lookup
#		ltype - lookup type (only -dns- at the moment)
#
sub GetLookupValue {

	my ($lattr, $ltype) = @_;

	return undef if ($ltype ne 'dns');	# only dns

	if ($lookupcount == 0) {
		# Create a DNS resolver object
		if ($config->dns ne "<undef>") {	# get dns server lsit from command line?
			my @arr = split /,/, $config->dns;
			$lupdns = Net::DNS::Resolver->new( nameservers => \@arr );
		} else {
			$lupdns = Net::DNS::Resolver->new ();
		}
		# initialize cache
		%dnscache = ();
	}

	$lookupcount++;

	# We do only reverse lookup
	return undef if (!($lattr =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/));
	# Do we have a cache hit here ?
	return $dnscache {$lattr} if exists $dnscache {$lattr};

	my $query = $lupdns->send ($lattr);	# Send a DNS query
	if ($query) {
		foreach my $rr ($query->answer) {
			next unless $rr->type eq "PTR";	# Only interested in first PTR record
			$dnscache {$lattr} = $rr->rdatastr;
			return $rr->rdatastr;
		}
	}
 
	return ($dnscache {$lattr} = ''); # no success with query, return cache as well
}

###############################
# WriteDetailsPages : 
#	Generates html pages recursively
#	Five parameters -
#		dim - current dimension
#		parent - pointer to count tables for the current level
#		parentsum - number of occurences for parent level (sum)
#		repdir - Current report directory
#		textarr - includes names and values of previous level table entries
#
sub WriteDetailsPages {

	my ($dim, $pparent, $repdir, @textarr) = @_;

	my $parent = $pparent->{"ref"};
	my $parentcount = $pparent->{"count"};
	my $parentsize = $pparent->{"size"} / 1024;

	ProcessVerboseInfo ('page', $pagecount) if $verbose;
	
	mkdir $repdir if (! -e $repdir);	# Make current report directory
	my $prevdir = cwd;
	chdir $repdir;
	
	# Generate file name for details pages at 1st level
	if ($dim == 1) {
	
		open my $fp, "> $indexpage" or die $dieopenmsg;
	
		print $fp $htmlheader;
		print $fp $q->hr;
		print $fp '<table border="1">';

		my $onlyfirst = 0;
	
		my $nr = 0;
	
		foreach my $currattr (@dim) {

			# Make link if qualified (only 1st element for subset vectors)
			my $link = (($subsetvector && !$onlyfirst) || (!$subsetvector)) ? 
				($q->a ({href=>"$currattr/index.html"}, $currattr)) : 
				($q->font({color=>"808080"}, $currattr));
		
			$onlyfirst = 1 if ($subsetvector);

			print $fp "<tr>" if (($nr % 4) == 0);
			print $fp "<td>", $q->td ( $q->div ({align=>"center"}, $q->h3 ($link))), "</td>";
			print $fp "</tr>" if ((($nr+1) % 4) == 0);

			$nr++;
		}

		print $fp "</table>";	
		
		# Write unprocessed attributes too (may be anoption in the future
		print $fp $q->hr;
		print $fp '<table border="1">';
		$nr = 0;

		foreach my $currattr (@DimOrg) {
				
			next if grep (/$currattr/, @dim);	# Skip processed attributes
			
			my $link = $q->font({color=>"808080"}, $currattr);
			print $fp "<tr>" if (($nr % 8) == 0);
			print $fp "<td>", $q->td ( $q->div ({align=>"center"}, $link)), "</td>";
			print $fp "</tr>" if ((($nr+1) % 8) == 0);

			$nr++;
		}
	
		print $fp "</table>";	
		print $fp $htmltrailer;	
		close $fp or die $dieclosemsg;	
	}
	
	$dim++;
	
	foreach my $currattr (keys %$parent) {
	
		# Number of entries at next level, no graph will be produced if only 1 entry
		my $nentry = scalar keys %{$parent->{$currattr}};
		
		# Generate file name for details pages
		my $currdir = cwd;
		mkdir "$currattr" if (! -e "$currattr");	# make directory for current dimension
		chdir $currattr;
		
		open my $fp2, "> $indexpage" or die $dieopenmsg;
		print $fp2 $htmlheader;
		
		foreach my $elem (@textarr) { print $fp2 $q->p ($q->h3 ($elem)) };
		print $fp2 $q->hr;

		print $fp2 
			$q->h3 ("Top graph for ", $q->font( {color=>"#800000"}, $q->b (ucfirst ($currattr)))),
			"<table border='1'>";

		# Generate html output for Top N graph if number of entries ae more than 1 
		if ($nentry > 1) {

			print $fp2 "<tr>";
			print $fp2 $q->td ($q->img ({src=>"$currattr-count.png"}));

			print $fp2 
				$q->td (), 
				$q->td ($q->img ({src=>"$currattr-size.png"}))
			  unless ($parentsize == 0);

			print $fp2 "</tr>";
		}
		
		print $fp2 
			"</table><hr><table border='1'><tr>";		

		print $fp2
			$q->td ($q->b ("$currattr")),
			$q->td ($q->b ("Count")),
			$q->td ($q->b ("ratio (%)")),
			$q->td ();

		print $fp2
			$q->td ({-width => "30"}),
			$q->td ($q->b ("$currattr")),
			$q->td ($q->b ("Size KB")),
			$q->td ($q->b ("ratio (%)"))
		  unless ($parentsize == 0);

		print $fp2 "</tr>";

		my $currattrval = $parent->{$currattr};
		
		# Generate html output for table entries sorted by sum of occurences
		
		# initialize graph array
		my @graphdata = my @graphdata3 = undef;
		my $graphxn = 0;

		# Sort count & sizes
		my @sortedcount = ();
		my @sortedsize = ();

		# Date related attributes are sorted by their values to get trends
		if ($currattr =~ /month/ || $currattr =~ /day/ || $currattr =~ /hour/) {
			@sortedcount  = sort {$a <=> $b} keys %$currattrval;
			@sortedsize = sort {$a <=> $b} keys %$currattrval
				unless ($parentsize == 0);
		} else { # Other attributes are sorted by count and size if supported
			@sortedcount  = sort { $currattrval->{$b}{"count"} <=> $currattrval->{$a}{"count"} } keys %$currattrval;
			@sortedsize = sort { $currattrval->{$b}{"size"}  <=> $currattrval->{$a}{"size"} }  keys %$currattrval
				unless ($parentsize == 0);
		}
 
		my $xcount = 
			(scalar @sortedcount < $config->MAX_X_COUNT) ? scalar @sortedcount : $config->MAX_X_COUNT; # Throttling output
		
		for my $i ( 0 .. $xcount - 1) {

			my $currattr2 = $sortedcount [$i];
			my $currattr3 = $sortedsize [$i];

			my $newparent = $currattrval->{ $currattr2 };
			my $newparent3 = $currattrval->{ $currattr3 }
				unless ($parentsize == 0);
			my $repdir2 = $newparent->{"directory"}; # Get directory name for attribute value

			# Take lookup value into consideration	
			my $lupval = $newparent->{"lookup"};
			my $lcurrattr2 = (defined $lupval && $lupval ne "") ? $lupval : $currattr2; 
			my $lcurrattr3 = (defined $lupval && $lupval ne "") ? $lupval : $currattr3
				unless ($parentsize == 0); 
		
			my $count2 = $newparent->{"count"};		
			my $ratio2 = sprintf "%.2f", ($count2 / $parentcount) * 100;
			my $count3 = sprintf "%.2f", $newparent3->{"size"} / 1024
				unless ($parentsize == 0);
			my $ratio3 = sprintf "%.2f", ($newparent3->{"size"} / 1024 / $parentsize) * 100
				unless ($parentsize == 0);

			my $link = '';
			if ($dim <= $ndimension) {
				foreach my $tmpattr (keys %{$newparent->{'ref'}}) {
					$link .= $q->a ({href=>"$repdir2/$tmpattr/index.html"}, $tmpattr). "&nbsp;";
				}
			}
			
			print $fp2 "<tr>";

			print $fp2
				$q->td ($lcurrattr2), 
				$q->td ($count2),
				$q->td ($ratio2),
				$q->td ($link);

			print $fp2
				$q->td (),
				$q->td ($lcurrattr3),
				$q->td ($count3), 
				$q->td ($ratio3)
			  unless ($parentsize == 0);			
			
			print $fp2 "</tr>";
				
			# Generate graph data
			# Special treatment of trend attributes month, day and hour
			my $topcount =  ($currattr eq "month") ? 13 :
					($currattr eq "day") ? 32 :
					($currattr eq "hour") ? 25 :
					$config->MAXTOPCOUNT;
				
			if ($nentry > 1 && $graphxn < $topcount) {
				
				$graphdata [0][$graphxn] = GraphString ($lcurrattr2);
				$graphdata [1][$graphxn] = $newparent->{"count"};

				if ($parentsize != 0) {
					$graphdata3 [0][$graphxn] = GraphString ($lcurrattr3);
					$graphdata3 [1][$graphxn] = $newparent3->{"size"}
				}
	
				$graphxn++;
			}
			
			next if ($dim > $ndimension);
			
			push @textarr, "$currattr : $lcurrattr2";
			my $currdir2 = cwd;
			WriteDetailsPages ($dim, $newparent, $repdir2, @textarr);
			pop @textarr;
			chdir $currdir2;
						
		}
		
		# Print sum information at the end
		print $fp2 
			"<tr>",
			$q->td ($q->b("SUM")), 
			$q->td ($q->b($parentcount)),
			$q-> td(''),
			$q-> td('');

		print $fp2
			$q->td (),
			$q->td ($q->b("SUM")), 
			$q->td ($q->b(sprintf ("%.2f MB", $parentsize / 1024)))
		  unless ($parentsize == 0);

		print $fp2
			"</tr>",
			"</table>",
			$htmltrailer;

		close $fp2 or die $dieclosemsg;
		$pagecount++;
		
		# generate graph
		if ($nentry > 1) {
			GenerateGraph ('count', $currattr, $graphxn, 'dblue', \@graphdata);
			GenerateGraph ('size',  $currattr, $graphxn, 'dgreen', \@graphdata3)
				unless ($parentsize == 0);
		}
		
		chdir $currdir;
				
	}
	
	chdir $prevdir;

}

###############################
# GenerateHtmlHeader : 
#	Generates a standard html heeader
#	Optional two arguments for summary information
#	sumtitle, suminfo
#
sub GenerateHtmlHeader {

	my ($sumtitle, $suminfo) = @_;

	my $header = $q->start_html(
		-title=>"LOGREP - Log file reporting system",
		-style=>{-code=>$styleCode});

	return $header if (!(defined $sumtitle && defined $suminfo));

	#Generate summary header
	my $summaryheader = $q->h1 ($sumtitle);	
	
	$summaryheader .=
		$q->p (	
			$q->b ("Filename: "),
			"$logfilename ($n lines, from $startdate to $enddate)"
		);
	
	foreach my $summary (keys %$suminfo) {

		$summaryheader .=
			$q->p (
				$q->b ($summary),
				$suminfo->{$summary}
			);
	}
	
	$summaryheader .=
		$q->p (
			"Number of analyzed dimensions is ", $q->b ("$ndimension."),
	);
	
	return "$header\n$summaryheader";

}

###############################
# GenerateHtmlTrailer : 
#	Generates a standard html trailer
#	sumtitle, suminfo
#
sub GenerateHtmlTrailer {

	return
		$q -> hr . 
		$q->font ( {size => "-2"},
			"This report is created on ", $q->b (scalar localtime), " by ",
			$q->a ({href => "http://logrep.sourceforge.net"}, "Logrep")
		) .
		$q->end_html;

}

###############################
# GenerateGraph : 
#	Generates a graph
#	grtype - count or size
#	grattr - graph attribute
#	grn - number of graph elements
#	grcolor - graph color
#	ptitle - data array pointer

sub GenerateGraph {

	my ($grtype, $grattr, $grn, $grcolor, $pdata) = @_;

	my $graph = GD::Graph::bars->new($config->GRAPH_SIZE_X, $config->GRAPH_SIZE_Y) 
		or die GD::Graph->error;

	$graph->set( 
		x_labels_vertical => "1",
		y_label => $grtype, 
		title => "$grattr / $grtype - Top $grn", 
		dclrs => [$grcolor]) 
	  or die $graph->error;		

	my $gd = $graph->plot($pdata) 
		or die $graph->error;
			
	open my $imgfp, "> $grattr-$grtype.png" 
		or die "Problems with creating graph image - $!\n"; 
	binmode $imgfp;
	print $imgfp $gd->png if (defined $gd);
	close $imgfp;
}

###############################
# GraphString :
#	Formats graph strings longer than maximum allowed length
#	Parameter: String to trim
#	Return : Formatted string
sub GraphString {

	my $string = shift;

	return $string if (length ($string) <= $config->MAXGRAPHSTRLEN);
	
	$n = ($config->MAXGRAPHSTRLEN - 4) / 2;
	
	return sprintf '%s[..]%s', substr ($string, 0, $n), substr ($string, -$n);
}

###############################
# CheckSubsetList :
#   Check if a subset list is specified and verifies contents of the subset list. 
#  If everything is Ok, subset list becomes the new complete list.

sub CheckSubsetList {

	my $subsetres = 0;	# No subset

	if ($subsetargs ne "<undef>") {{	# double brackets for last command!!
	
		$subsetres = 1; # Subset list is specified. Must be verified!!
	
		my $ndim = @dim; # get number of parameters in the original complete list
	
		my @arr = split /,/, $subsetargs, $ndim;
	
		my $narr = @arr;
		last if ($narr == 0); # Don't continue if subset list is empty!
		
		# Check if subset is a really subset of the complete list
		my $nok = 0;
		foreach my $x (@arr) {

			# Process composite attributes, number of attributes as a random limit
			my @comparr = split /\./, $x;
			my $compnarr = @comparr;
			my $compnok = 0;

			foreach my $compx (@comparr) {
				
 				$compnok++ if (grep (/$compx/, @dim));

			}

			$nok++ if ($compnok == $compnarr);

		};

		last if ($narr != $nok); # Don't continue if subset list couldn't be verified
		
		@dim = @arr; # New array is qualified and replaces the complete list

		# Take user specified dimension into account if specified
		my $maxdim = scalar @dim;

		if ($subsetvector) {
			$ndimension = $maxdim;
			print "Subsetvector: Number of dimensions is automatically set to number of attributes.\n"
				if $verbose;
		} elsif ($maxdim < $ndimension) {
			$ndimension = $maxdim;
			print "New number of dimensions is $maxdim.\n"
				if $verbose;
		}
		
		$subsetres = 0; # subset is a complete list now!
	}}
	
	if ($subsetres) { # Wrong subset!!
	
		my $lststr = "";
		foreach my $x (@dim) {$lststr .= "$x,"}; 
		chop $lststr;
		
		print "Wrong subset parameter list : $subsetargs.\nA complete list of\n  $lststr\n is assumed\n";
	}
}

###############################
# VerifyParameters :
# 	Verifies command line parameters
sub VerifyParameters {
	
	my $nerr = 0;
	
	if (!(-e $logfilename)) { print STDERR "Log file $logfilename does not exist.\n"; $nerr++; };
	if (!($ndimension =~ /^\d+$/)) { print STDERR "Dimension ($ndimension) is not an integer.\n"; $nerr++ };
	
	exit 1 if ($nerr > 0);	

}

###############################
# CheckLookupList :
#   Check if a lookup list is specified and verifies contents of the that list. 

sub CheckLookupList {

	if ($lookupargs ne "<undef>") {{	# double brackets for last command!!
	
		my @arr = split /[,:]/, $lookupargs;
		
		# Check if lookup pairs are valid for this run
		my $lcnt = scalar @arr;

		for (my $i; $i < $lcnt; $i+=2) {
			
			my $lattr = $arr [$i];
			my $luptype = $arr [$i+1];
			
			print "Lookup pair : $lattr, $luptype\n" if $verbose;

			# Attribute must be available and we have currently only 'dns' as lookup
			if (!(grep (/$lattr/, @dim) && $luptype eq "dns")) {
				print "Wrong lookup list : $lookupargs.\n";
				return 1;
			}

			$lookup {$lattr} = $luptype;	# populate lookup hash
			
		 };
		
	}}
	
}

# Module end!!
1;

